program PCMAGXI;
uses  SysUtils, Math;

const   Mmax = 10;
        Mmin = 1;
        Nmax = 100;
        Nmin = 1;
        Hmax = 100;
        Hmin = 0;

type    TBlock = record
          Height,               // block height
          Si,                   // southernmost tip
          SL,                   // length of southern profile
          Wi,                   // westernmost tip
          EL : Integer;         // length of eatern profile
        end;

var f, g : text; // input & output files
    S : String;
    M, N, NHeight, OHeight, i, ci, H : Integer;
    SouthMap, EastMap : array[-1..Nmax] of Integer;
    Used    : array[0..Nmax-1,0..Nmax-1] of Integer; // counters
    Block : array[0..Hmax] of TBlock; // blocks sorted by height
    PassedS, PassedE : array[0..Nmax] of Boolean;

// returns the first number in S and removes it from S

function First(Var S : String) : Integer;
Var i : Integer;
begin
  i:=Pos(' ',S);
  if i=0 Then i:=Length(S)+1;
  Result:=StrToIntDef(Copy(S,1,i-1),0);
  Delete(S,1,i);
end;

// finds the block AHeight or inserts such into Block and returns its index

function NewHeight( AHeight : Integer) : Integer;
var i, j : Integer;
begin
  i:=0;j:=H;Result:=0;
  While i<j Do
    Begin
      Result:=(i+j) div 2;
      if Block[Result].height<AHeight
          Then i:=Result+1
          Else if Block[Result].Height>AHeight Then j:=Result-1
                                                 Else Break;
   End;

 if i=j Then Result:=i;

 if Block[Result].height<>AHeight Then
    Begin
      Inc(H);
      if Block[Result].height<AHeight Then Inc(Result);
      for i:=H downto Result+1 do Block[i]:=Block[i-1];
      Block[Result].Height:=AHeight;
      Block[Result].Si:=N-M;
      Block[Result].Wi:=N-M;
      Block[Result].SL:=-1;
      Block[Result].EL:=-1;
   End;
end;

// updates Used when a block is Situate-d (S=1), or removed (S=-1)

procedure UsedUpdate(X,Y,S : Integer);
var i,j : Integer;
begin
  for i:=Max(0,X+1-M) to Min(X+M-1,N-1) do
    for j:=Max(0,Y+1-M) to Min(Y+M-1,N-1) do Inc(Used[i,j],S);
end;

// tries to situate a block at X,Y and returns TRUE if possible

function Situate(X,Y : Integer) : Boolean;
begin
  Result:=False;
  if (X<0) or (Y<0) or (X>N-M) or (Y>N-M) or (Used[X,Y]>0) Then Exit;

  UsedUpdate(X,Y,1);
  Result:=True;
end;

// tries to place Block[1],..,Block[NH] in Used and returns TRUE if possible

function Place(NH : Integer) : Boolean;
var i,j : Integer;
begin
  Result:=True;
  if NH=0 Then Exit;

  With Block[NH] Do
    if (SL=M) and (EL=M)
        Then Result:=Place(NH-1)
        Else Begin
               i:=Wi;
               Wi:=i+SL-M-1;
               if (Wi<0) or (SL<1) Then Wi:=-1;
               repeat
                 Inc(Wi);
                 While (Wi<=i) and ((SouthMap[Wi]<height) or (SouthMap[Wi+M-1]<height)) do
                   if SouthMap[Wi+M-1]<height Then Wi:=Wi+M
                                                   Else Inc(Wi);
                 j:=Si;
                 Si:=j+EL-M-1;
                 if (Si<0) or (EL<1) Then Si:=-1;
                 repeat
                   Inc(Si);
                   While (Si<=j) and ((EastMap[Si]<height) or (EastMap[Si+M-1]<height)) do
                     if EastMap[Si+M-1]<height Then Si:=Si+M
                                               Else Inc(Si);

                   if Situate(Wi,Si) Then
                       Begin
                         If Place(NH-1) Then Exit;
                         UsedUpdate(Wi,Si,-1);
                       End;
                 until (Si>=j);

                 Si:=j;
               until (Wi>=i);
               Wi:=i;

               Result:=False;
             End;
end;

// Writes 'NO' if no placement is possible

procedure NO;
begin
  Close(f);
  WriteLn(g,'NO');
  Close(g);
  Halt;
end;

// Main

BEGIN
  Assign(f,'BLOCK.IN');
  {$i-}Reset(f);{$i+}
  if IORESULT<>0 Then Halt;

  Assign(g,'BLOCK.OUT');
  Rewrite(g);

  if not Eof(f) Then ReadLn(f,S)
                Else NO;

  N:=First(S);
  if not (N in [Nmin..Nmax]) Then No;
  M:=First(S);
  if (M>N) or not (M in [Mmin..Mmax]) Then NO;
  H:=0;

  Block[0].Height:=0;

  if not Eof(f) Then ReadLn(f,S)
                Else NO;

// Do the southern profile

  OHeight:=0;
  for i:=0 to N Do
        Begin
          NHeight:=First(S);
          PassedS[i]:=NHeight=0;
          if not (NHeight in [Hmin..Hmax]) Then NO;
          SouthMap[i]:=NHeight;

          if NHeight>OHeight Then
             Begin
               if i+M>N Then NO;
               ci:=NewHeight(NHeight);
               if Block[ci].SL>0 Then NO;
               Block[ci].Wi:=i;
               Block[ci].SL:=M;
             End
             Else
          if NHeight<OHeight Then
             Begin
               if i<M Then NO;
               Block[ci].Wi:=i-M;
               Block[ci].SL:=M;
               if NHeight>0 Then Begin
                                   ci:=NewHeight(NHeight);
                                   if Block[ci].SL>0 Then NO;
                                   Block[ci].Wi:=i;
                                   Block[ci].SL:=1;
                                 End;
             End
             Else if (NHeight>0) and (Block[ci].SL<M) Then Inc(Block[ci].SL);

           OHeight:=NHeight;
        End;

  if not Eof(f) Then ReadLn(f,S)
                Else NO;

// Do the eastern profile

  OHeight:=0;
  for i:=0 to N Do
        Begin
          NHeight:=First(S);
          PassedE[i]:=NHeight=0;
          if not (NHeight in [Hmin..Hmax]) Then NO;
          EastMap[i]:=NHeight;

          if NHeight>OHeight Then
             Begin
               if i+M>N Then NO;
               ci:=NewHeight(NHeight);
               if Block[ci].EL>0 Then NO;
               Block[ci].Si:=i;
               Block[ci].EL:=M;
             End
             Else
          if NHeight<OHeight Then
             Begin
               if i<M Then NO;
               Block[ci].Si:=i-M;
               Block[ci].EL:=M;
               if NHeight>0 Then Begin
                                   ci:=NewHeight(NHeight);
                                   if Block[ci].EL>0 Then NO;
                                   Block[ci].Si:=i;
                                   Block[ci].EL:=1;
                                 End;
             End
             Else if (NHeight>0) and (Block[ci].EL<M) Then Inc(Block[ci].EL);

           OHeight:=NHeight;
        End;

  for i:=0 to N-1 do
    for ci:=0 to N-1 do Used[i,ci]:=0;

// find a placement

  for i:=1 to H Do
   With Block[i] Do
    Begin
      if (SL=M) and (EL=M) Then
          if not Situate(Wi,Si) Then NO;
    End;


  if not Place(H) Then NO;

// verify placement

  for i:=1 to H Do
   With Block[i] Do
    for ci:=0 to M-1 do
    begin
      if SouthMap[Wi+ci]<Height Then No
                                Else if SouthMap[Wi+ci]=Height Then PassedS[Wi+ci]:=True;
      if EastMap[Si+ci]<Height  Then No
                                Else if EastMap[Si+ci]=Height  Then PassedE[Si+ci]:=True;
    end;

  for i:=0 to N-1 do
    if not (PassedS[i] and PassedE[i]) Then NO;

  Close(f);

  WriteLn(g,IntToStr(H));

  for i:=1 to H Do
    With Block[i] Do
      WriteLn(g,IntToStr(Wi)+' '+IntToStr(Si)+' '+IntToStr(Height));

  Close(g);
end.
