/*
 Blocks.cpp - solution of second round of PC Magazine Contest.

  Author: Ivan Anev
  Age: 16
  Phone: 751178
  City: Sofia
*/

#include <iostream.h>
#include <fstream.h>
#include <process.h>

class CBlock {
public:
	int x, y, h;
};

class CBlocks {
private:
	int n, m, blocks, count;
	int ** layerAND;
	int * heights, * east, * south, *coverS, * coverE, * layerS, * layerE;
	CBlock * found;
private:
	void ReadData ( void );
	void CreateLayers ( void );
	void GetHeights ( void );
	void AndLayers ( void );
	void SubLayers ( void );
	void Solve ( int index );
	void WriteSolution ( void );
	void EastAndSouth ( int h, int c );
	int InBlock ( int x, int y );
	void AddBlock ( int x, int y, int h );
	void AddCover ( int x, int y, int c );
	int BlockCovered ( int h );
	void RemoveBlock ( void );
	int GetLeftmost ( int h );
	int GetRightmost ( int h );
	int GetTopmost ( int h );
	int GetBottommost ( int h );
	static int NumInArray ( int num, int * array, int size );
	inline static int min ( int a, int b );
	inline static int max ( int a, int b );
public:
	void Start ( void );
	~CBlocks ( void );
};

void CBlocks::Start ( void ) {
	ReadData ();
	GetHeights ();
	Solve ( 0 );
	
	ofstream output ( "BLOCK.OUT", ios::out );
	output << "NO";
}

void CBlocks::ReadData ( void ) {
	ifstream input ( "BLOCK.IN" );
	int i;

	input >> n >> m;
	CreateLayers ();
	for ( i = 0; i < n; ++i ) {
		input >> south[i];
	}
	for ( i = 0; i < n; ++i ) {
		input >> east[i];
	}
}

void CBlocks::CreateLayers ( void ) {
	int i;

	count = 0;
	
	east = new int [n];
	south = new int [n];
	coverS = new int [n];
	coverE = new int [n];
	layerS = new int [n];
	layerE = new int [n];
	found = new CBlock [n];
	layerAND = new int * [n];

	for ( i = 0; i < n; ++i ) {
		layerAND[i] = new int [n];
		coverS[i] = coverE[i] = layerS[i] = layerE[i] = 0;
	}
}

void CBlocks::GetHeights ( void ) {
	int i, j;

	heights = new int [100];

	for ( i = 0, blocks = 0; i < n; ++i ) {
		if ( south[i] && !NumInArray ( south[i], heights, blocks ) )
			heights[blocks++] = south[i];
		if ( east[i] && !NumInArray ( east[i], heights, blocks ) )
			heights[blocks++] = east[i];
	}

	for ( i = 0; i < blocks - 1; ++i )
		for ( j = i + 1; j < blocks; ++j )
			if ( heights[i] < heights[j] ) {
				heights[i] ^= heights[j];
				heights[j] ^= heights[i];
				heights[i] ^= heights[j];
			}
}

void CBlocks::AndLayers ( void ) {
	int i, j;

	for ( i = 0; i < n; ++i )
		for ( j = 0; j < n; ++j )
			layerAND[i][j] = layerS[i] && layerE[j];
}

void CBlocks::SubLayers ( void ) {
	int i, j, k;

	for ( k = 0; k < count; ++k )
		for ( i = found[k].x; i < found[k].x + m; ++i )
			for ( j = found[k].y; j < found[k].y + m; ++j )
					layerAND[i][j] = 0;

}

void CBlocks::Solve ( int index ) {
	if ( index == blocks ) {
		WriteSolution ();
		exit ( 0 );
	}
	else {
		int i, j, lm, rm, tm, bm;

		EastAndSouth ( heights[index], 1 );
		AndLayers ();
		SubLayers ();

		lm = GetLeftmost ( heights[index] );
		rm = GetRightmost ( heights[index] );
		tm = GetTopmost ( heights[index] );
		bm = GetBottommost ( heights[index] );

		for ( i = lm; i < rm; ++i )
			for ( j = tm; j < bm; ++j )
				if ( layerAND[i][j] && InBlock ( i, j ) ) {
					AddBlock ( i, j, heights[index] );
					AddCover ( i, j, 1 );
					if ( BlockCovered ( heights[index] ) )
						Solve ( index + 1 );
					RemoveBlock ();
					AddCover ( i, j, 0 );
				}

		EastAndSouth ( heights[index], 0 );
	}
}

void CBlocks::EastAndSouth ( int h, int c ) {
	int i;

	for ( i = 0; i < n; ++i ) {
		if ( south[i] == h )
			layerS[i] = c;
		if ( east[i] == h )
			layerE[i] = c;
	}
}

int CBlocks::InBlock ( int x, int y ) {
	int i, j;

	if ( x + m > n || y + m > n )
		return 0;

	for ( i = x; i < x + m; ++i )
		for ( j = y; j < y + m; ++j )
			if ( layerAND[i][j] == 0 )
				return 0;

	return 1;
}

void CBlocks::WriteSolution ( void ) {
	int i;
	ofstream output ( "BLOCK.OUT", ios::out );

	output << count << endl;
	for ( i = count - 1; i >= 0; --i )
		output << found[i].x << ' ' << found[i].y << ' ' << found[i].h << endl;
}

void CBlocks::AddBlock ( int x, int y, int h ) {
	found[count].x = x;
	found[count].y = y;
	found[count].h = h;
	++count;
}

void CBlocks::AddCover ( int x, int y, int c ) {
	int i;

	for ( i = x; i < x + m; ++i )
		coverS[i] = c;

	for ( i = y; i < y + m; ++i )
		coverE[i] = c;

}

int CBlocks::BlockCovered ( int h ) {
	int i;

	for ( i = 0; i < n; ++i ) {
		if ( south[i] == h && coverS[i] == 0 )
			return 0;
		if ( east[i] == h && coverE[i] == 0 )
			return 0;
	}

	return 1;
}

void CBlocks::RemoveBlock ( void ) {

	--count;

}

int CBlocks::NumInArray ( int num, int * array, int size ) {
	int i;

	for ( i = 0; i < size; ++i )
		if ( array[i] == num )
			return 1;

	return 0;
}

int CBlocks::GetLeftmost ( int h ) {
	int i;

	for ( i = 0; i < n && south[i] != h; ++i );
	if ( i < n && i >= m )
		return ( i - m );
	return 0;
}

int CBlocks::GetRightmost ( int h ) {
	int i;

	for ( i = n - 1; i >= 0 && south[i] != h; --i );
	if ( i >= 0 && i + m <= n )
		return ( i + m );
	return n;
}

int CBlocks::GetTopmost ( int h ) {
	int i;

	for ( i = 0; i < n && east[i] != h; ++i );
	if ( i != n && i >= m )
		return ( i - m );
	return 0;
}

int CBlocks::GetBottommost ( int h ) {
	int i;

	for ( i = n - 1; i >= 0 && east[i] != h; --i );
	if ( i >= 0 && i + m < n )
		return ( i + m );
	return n;
}

int CBlocks::min ( int a, int b ) {
	return ( ( a < b ) ? ( a ) : ( b ) );
}

int CBlocks::max ( int a, int b ) {
	return ( ( a > b ) ? ( a ) : ( b ) );
}

CBlocks::~CBlocks ( void ) {
	int i;

	delete [] east;
	delete [] south;
	delete [] heights;
	delete [] coverS;
	delete [] coverE;
	delete [] found;
	
	for ( i = 0; i < n; ++i ) {
		delete layerAND[i];
	}

	delete [] layerS;
	delete [] layerE;
	delete [] layerAND;
}

void main ( void ) {
	CBlocks solution;
	solution.Start ();
}